//Streaming as JSON
//TODO: Rewrite all writers to Writable Streams : https://nodejs.org/dist/latest-v8.x/docs/api/stream.html#stream_writable_streams
class WriterJSON {
    constructor(res) {
        this.res = res;
        this.sep = '';
    }

    writeHeader() {
        this.res.setHeader('Content-Type', 'application/json');
        this.res.write('{"result":[');
    }

    write(data) {
        return new Promise((resolve) => {
            for (let entity of data) {
                if (!this.sep) {
                    this.writeHeader();
                }
                this.res.write(this.sep + JSON.stringify(entity), false, 0);
                this.sep = ',';
            }
            resolve();
        });
    }

    end(error) {
        return new Promise((resolve, reject) => {
            if (!this.res.headersSent) {
                this.writeHeader();
            }
            this.res.write(']');
            if (error) {
                this.res.write(',"error":');
                this.res.write(JSON.stringify(`${error}`, null, 2));
            }
            this.res.write('}');
            this.res.end();
            if (error) {
                return reject(error);
            }
            resolve();
        });
    }
}

module.exports = WriterJSON;
